#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e14|e15|c2> <prof|debug> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}:${PRODUCTS}
}

function setup_and_verify()
{
    # this should not complain
    echo "Finished building ${package} ${pkgver}"
    setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
    echo "${package} is installed at ${JSONNET_FQ_DIR}"
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=jsonnet
origpkgver=v0_9_3
pkgver=${origpkgver}b
ssibuildshims_version=v1_04_04
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtar=jsonnet-${pkgdotver}.tar.gz

make_tarball_opts=("\${product_dir}" "\${package}" "\${pkgver}" "\${fullqual}")

get_this_dir
get_ssibuildshims
source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) cflg="-g -O0";;
prof)  cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer";;
* )
   echo "ERROR: please specify debug or prof"
   usage
   exit 1
;;
esac

if [[ "${basequal}" == e1[0245] ]]
then
  cc=gcc
  cxx=g++
  cxxflg="${cflg} -std=c++14"
elif [[ "${basequal}" == c2 ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17"
else
  ssi_die "Qualifier $basequal not recognized."
fi

mkdir -p ${pkgdir}/bin
if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   exit 1
fi
mkdir -p ${pkgdir}/lib
if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   exit 1
fi
mkdir -p ${pkgdir}/include
if [ ! -d ${pkgdir}/include ]
then
   echo "ERROR: failed to create ${pkgdir}/include"
   exit 1
fi

fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

cd ${pkgdir} || ssi_die "Unable to cd to ${pkgdir}"
tar xf ${tardir}/${pkgtar} || ssi_die "Unable to unwind ${tardir}/${pkgtar} into ${PWD}"

# We only make an optimized build, because jsonnet does not support
# anything else (and nobody would really want it anyway).
cd ${pkgdir}/jsonnet-${pkgdotver} || ssi_die "Unable to cd to ${pkgdir}/jsonnet-${pkgdotver}"

#patch
patch -b -p1 < ${patchdir}/jsonnet-${pkgdotver}.patch

set -x

export FC=gfortran
export CC=${cc}
export CXX=${cxx}
make CFLAGS_LOCAL="${cflg}" CXXFLAGS_LOCAL="${cxxflg}" || ssi_die "Failed in 'make'"
make CFLAGS_LOCAL="${cflg}" CXXFLAGS_LOCAL="${cxxflg}" libjsonnet.so || ssi_die "Failed in 'make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" libjsonnet.so'"
make CFLAGS_LOCAL="${cflg}" CXXFLAGS_LOCAL="${cxxflg}" libjsonnet++.so || ssi_die "Failed in 'make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" libjsonnet++.so'"
# this is the "install" step
cp -p ${pkgdir}/jsonnet-${pkgdotver}/jsonnet ${pkgdir}/bin/ || exit 1
cp -p ${pkgdir}/jsonnet-${pkgdotver}/include/*.h ${pkgdir}/include/ || exit 1
cp -p ${pkgdir}/jsonnet-${pkgdotver}/*.so ${pkgdir}/lib/ || exit 1
# make clean removes EVERYTHING
make clean || ssi_die "Failed to clean"

set +x

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

setup_and_verify

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
fi

exit 0
